package kamera.teszt;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import android.content.Context;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.PreviewCallback;
import android.hardware.Camera.ShutterCallback;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceHolder.Callback;
import android.view.SurfaceView;

/**
 * 
 * Kamera hasznlatnak demonstrlsa
 * ==================================
 * 
 * A kamerbl jv kp nagy adatmennyisgnek szmt egy telefon letben.
 * Emiatt a kp megjelentse a lehet legegyszerbb s legrvidebb ton trtnik.
 * Kamera kpt sszektjk egy megjelent fellettel.
 * - A SurfaceView biztost egy felletet. 
 * - Ezt a felletet biztostanunk  kell a kamernak. (setPreviewDisplay <-- SurfaceHolder)
 * 
 */

public class CameraPreview extends SurfaceView implements Callback, AutoFocusCallback, PreviewCallback {

	SurfaceHolder holder;	// birtokos
	private Camera cam;
	
	
	// Kamera belltsok -----------------------------
	int width 			= 320,
		height			= 240;
	int pictureWidth 	= 1600,
		pictureHeight 	= 1200;
	int fps				= 15;
	String colorEffect 	= Camera.Parameters.EFFECT_MONO;
	String focusMode   	= Camera.Parameters.FOCUS_MODE_AUTO;
	String sceneMode	= Camera.Parameters.SCENE_MODE_SPORTS;
	String whiteBalance = Camera.Parameters.WHITE_BALANCE_DAYLIGHT;
	
	// ----------------- KONSTRUKTOR ------------------
	public CameraPreview(Context context) {
		super(context);		
		
		 holder = getHolder();					// A SurfaceView biztostja 
		 holder.setFixedSize(width, height);	// Mret megadsa		 
		 holder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);	// Bufferels		 
		 holder.addCallback(this);		// Esemnyek: Surface ltrejn, vltozik, bezr
		 
	}

	// ------------------ SURFACEHOLDER ------------------
	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		// Kamera elindtsa		
		cam = Camera.open();
		try {
			cam.setPreviewCallback(this);	// Enzeti kp el-feldolgozsa
			cam.setPreviewDisplay(holder);	// belltani a felletkezelt			
		} catch (IOException e) {
			surfaceDestroyed(holder);
		}
	}
	
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {
		
		if(cam != null){
			cam.stopPreview();				// Ha netn ment volna korbban
			Camera.Parameters params = cam.getParameters();			
			
			// ColorEffect: tmogats ellenrzse
			if(params.getSupportedColorEffects() != null)
				if(params.getSupportedColorEffects().contains(colorEffect))
					params.setColorEffect(colorEffect);			
			
			// FPS
			if(params.getSupportedPreviewFrameRates() != null)
				if(params.getSupportedPreviewFrameRates().contains(fps))
					params.setPreviewFrameRate(fps);
					
			// Focus mode			
			if(params.getSupportedFocusModes() != null)
				if(params.getSupportedFocusModes().contains(focusMode))
					params.setFocusMode(focusMode);
			
			// Scene mode
			if(params.getSupportedSceneModes() != null)
				if(params.getSupportedSceneModes().contains(sceneMode))
					params.setSceneMode(sceneMode);
			
			// White balance
			if(params.getSupportedWhiteBalance() != null)
				if(params.getSupportedWhiteBalance().contains(whiteBalance))
					params.setWhiteBalance(whiteBalance);
			
			// Preview Size & Format
			params.setPreviewSize(width, height);
			params.setPictureSize(pictureWidth, pictureHeight);
			//params.setPreviewFormat(17); // NV21
			
			cam.setParameters(params);
			cam.startPreview();
			cam.autoFocus(this);
			
		}
	}
	

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		
		if(cam != null){
			cam.stopPreview();
			cam.setPreviewCallback(null);	// Forgats utni bug
			cam.release();
			cam = null;
		}
		
	}
	
	
	// ----------------- FOCUS CALLBACK -------------------	
	public void onAutoFocus(boolean success, Camera camera) {
		Log.d("NIK", "Fkuszlva!");
	}

	// ----------------- Preview Frame --------------------
	@Override
	public void onPreviewFrame(byte[] data, Camera camera) {
		// YUV formtum
		/* Kp kocka feldolgozsa */
	}
	
	// ------------------ Take picture --------------------
	public void takePictureNow(){
		if(cam != null) 
			cam.takePicture(shutter, raw, jpeg);
	}
	
	
	private ShutterCallback shutter = new ShutterCallback() {
		public void onShutter() {
			/* Kp ellve. */
		}
	};
	
	private PictureCallback raw = new PictureCallback() {
		public void onPictureTaken(byte[] data, Camera camera) {
			/* Elkszlt kp nyers adatai */
		}
	};
	
	private PictureCallback jpeg = new PictureCallback() {
		public void onPictureTaken(byte[] data, Camera camera) {
			FileOutputStream fos;
			try {
				fos = new FileOutputStream("/sdcard/cam.jpg");
				fos.write(data);
				fos.close();
			} catch (FileNotFoundException e) {
			} catch (IOException e) {}
		}
	};
	
	
	
	
	
	
	
}
